@@ -3,7 +3,7 @@
 """A wrapper for Paramiko that attempts to make ssh sessions easier to work with.  It also contains the
 shell_command function for running local shell scripts!
 """
-# Copyright (C) 2015-2018 Jesse Almanrode
+# Copyright (C) 2015-2020 Jesse Almanrode
 #
 #     This program is free software: you can redistribute it and/or modify
 #     it under the terms of the GNU Lesser General Public License as published by
@@ -20,14 +20,21 @@ shell_command function for running local shell scripts!
 from __future__ import print_function
 from collections import namedtuple, OrderedDict
 from getpass import getuser
+from past.builtins import basestring
 import logging
 import os
 import paramiko
 import socket
+import sys
 
 # Privates
 __author__ = 'Jesse Almanrode (jesse@almanrode.com)'
 
+if sys.version_info[0] < 3:
+    raise Exception('Python2.x is no longer supported in this version of sshreader')
+elif sys.version_info[1] < 4:
+    raise Exception('Only Python 3.5 and later is supported in this version of sshreader')
+
 # Globals
 Command = namedtuple('Command', ['cmd', 'stdout', 'stderr', 'return_code'])
 
@@ -66,12 +73,11 @@ class SSH(object):
     :param keyfile: SSH private key file
     :param keypass: SSH private key password
     :param port: SSH port
-    :param timeout: SSH connection timeout in seconds
     :param connect: Initiate the connect
     :return: SSH connection object
     :raises: SSHException
     """
-    def __init__(self, fqdn, username=None, password=None, keyfile=None, keypass=None, port=22, timeout=30, connect=True):
+    def __init__(self, fqdn, username=None, password=None, keyfile=None, keypass=None, port=22, connect=True):
         if not keyfile:
             if len(paramiko.Agent().get_keys()) == 0:
                 if not all((username, password)):
@@ -80,19 +86,20 @@ class SSH(object):
         self.username = username
         self.password = password
         if keyfile:
+            if not isinstance(keyfile, basestring):
+                raise TypeError('expected %s for keyfile, got %s' % (str(basestring), str(type(keyfile))))
             self.keyfile = os.path.abspath(os.path.expanduser(keyfile))
         else:
             self.keyfile = keyfile
         self.keypass = keypass
         self.port = port
-        self.timeout = timeout
         self._connection = paramiko.SSHClient()
         self._connection.set_missing_host_key_policy(paramiko.AutoAddPolicy())
         if connect:
             self.__connect()
 
     def __str__(self):
-        return self.__dict__
+        return str(self.__dict__)
 
     def __enter__(self):
         if self.__alive() is False:
@@ -109,7 +116,7 @@ class SSH(object):
         :param dstfile: Path to the remote file
         :return: Result of paramiko.SFTPClient.put()
         """
-        if self.__alive() is False:
+        if not self.__alive():
             raise paramiko.SSHException("connection to %s not established" % (self.host,))
         sftp = paramiko.SFTPClient.from_transport(self._connection.get_transport())
         try:
@@ -125,7 +132,7 @@ class SSH(object):
         :param dstfile: Path to the local file
         :return: Result of paramiko.SFTPClient.get()
         """
-        if self.__alive() is False:
+        if not self.__alive():
             raise paramiko.SSHException("connection to %s not established" % (self.host,))
         sftp = paramiko.SFTPClient.from_transport(self._connection.get_transport())
         try:
@@ -196,9 +203,11 @@ class SSH(object):
         """
         return self.__connect()
 
-    def connect(self, failfast=True):
+    def connect(self, failfast=True, timeout=30):
         """Opens an SSH Connection
 
+        :param failfast: Test socket connection before attempting to connect
+        :param timeout: SSH connection timeout in seconds
         :return: True
         :raises: SSHException
         """
@@ -221,10 +230,10 @@ class SSH(object):
                     paramiko.SSHException('username and password or keyfile not provided')
         if self.keyfile:
             self._connection.connect(self.host, port=self.port, username=self.username, password=self.keypass,
-                                     key_filename=self.keyfile, timeout=self.timeout, look_for_keys=False)
+                                     key_filename=self.keyfile, timeout=timeout, look_for_keys=False)
         else:
             self._connection.connect(self.host, port=self.port, username=self.username, password=self.password,
-                                     timeout=self.timeout, look_for_keys=False)
+                                     timeout=timeout, look_for_keys=False)
         return True
 
     # Privatizing some of the functions so SSH can be subclassed
