@@ -1,7 +1,7 @@
 # coding=utf-8
 """ All the classes and functions that make sshreader tick
 """
-# Copyright (C) 2015-2018 Jesse Almanrode
+# Copyright (C) 2015-2020 Jesse Almanrode
 #
 #     This program is free software: you can redistribute it and/or modify
 #     it under the terms of the GNU Lesser General Public License as published by
@@ -33,12 +33,17 @@ import time
 
 __author__ = 'Jesse Almanrode (jesse@almanrode.com)'
 
+if sys.version_info[0] < 3:
+    raise Exception('Python2.x is no longer supported in this version of sshreader')
+elif sys.version_info[1] < 4:
+    raise Exception('Only Python 3.5 and later is supported in this version of sshreader')
+
+mpctx = multiprocessing.get_context('spawn')
 __cpuhardlimitfactor__ = 3
 __threadlimitfactor__ = 2
-_printlock_ = multiprocessing.Lock()
+_printlock_ = mpctx.Lock()
 log = logging.getLogger('sshreader')
 
-
 # Globals
 Command = namedtuple('Command', ['cmd', 'stdout', 'stderr', 'return_code'])
 
@@ -206,7 +211,8 @@ class ServerJob(object):
                 self.prehook.run(self)
             try:
                 self._conn = SSH(self.name, username=self.username, password=self.password, keyfile=self.key,
-                                 timeout=self.sshtimeout, port=self.ssh_port)
+                                 port=self.ssh_port, connect=False)
+                self._conn.connect(timeout=self.sshtimeout)
                 log.debug('%s: ssh connection established' % (self.name,))
             except Exception as errorMsg:
                 log.debug(str(errorMsg))
@@ -217,7 +223,10 @@ class ServerJob(object):
                     log.debug('%s: running prehook' % (self.name,))
                     self.prehook.run(self)
                 for cmd in self.cmds:
-                    result = self._conn.ssh_command(cmd, timeout=self.cmdtimeout, combine=self.combine_output)
+                    try:
+                        result = self._conn.ssh_command(cmd, timeout=self.cmdtimeout, combine=self.combine_output)
+                    except Exception as errorMsg:
+                        result = Command(cmd, '', str(errorMsg), 54)
                     log.debug('%s: %s' % (self.name, str(result)))
                     self.results.append(result)
                     self.status += result.return_code
@@ -225,6 +234,7 @@ class ServerJob(object):
                     log.debug('%s; running posthook' % (self.name,))
                     self.posthook.run(self)
                 self._conn.close()
+            finally:
                 self._conn = None  # So the ssh connection can be pickled!
             if self.posthook and self.posthook.ssh_established is False:
                 log.debug('%s; running posthook' % (self.name,))
@@ -244,7 +254,7 @@ def cpusoftlimit():
 
     :return: Integer
     """
-    cpu_count = multiprocessing.cpu_count()
+    cpu_count = mpctx.cpu_count()
     if cpu_count > 1:
         return cpu_count - 1
     else:
@@ -272,7 +282,7 @@ def threadlimit():
     """
     global __threadlimitfactor__
     assert isinstance(__threadlimitfactor__, int)
-    return multiprocessing.cpu_count() * __threadlimitfactor__
+    return mpctx.cpu_count() * __threadlimitfactor__
 
 
 def echo(*args, **kwargs):
@@ -314,19 +324,24 @@ def sshread(serverjobs, pcount=None, tcount=None, progress_bar=False):
         log.info('logging enabled: disabling progress bar')
         progress_bar = False
 
-    item_counter = multiprocessing.Value('L', 0)
     if progress_bar:
+        item_counter = mpctx.Value('L', 0)
         bar = ProgressBar(max_value=totaljobs)
     else:
+        item_counter = None
         bar = None
 
-    task_queue = multiprocessing.Queue(maxsize=totaljobs)
-    result_queue = multiprocessing.Queue(maxsize=totaljobs)
+    task_queue = mpctx.Queue(maxsize=totaljobs)
+    result_queue = mpctx.Queue(maxsize=totaljobs)
 
-    log.debug('filling input queue')
+    log.debug('filling task_queue')
     for job in serverjobs:
         task_queue.put(job)
 
+    while task_queue.empty():
+        log.debug('Waiting for task_queue to fill')
+        time.sleep(1)
+
     subs = list()  # Keep a list of processes/threads so we can join them later
 
     if pcount is None:
@@ -339,14 +354,14 @@ def sshread(serverjobs, pcount=None, tcount=None, progress_bar=False):
         log.info(u'spawning %d threads' % (tcount, ))
         # Start a thread pool
         for thread in range(tcount):
-            thread = threading.Thread(target=_sub_thread_, args=(task_queue, result_queue, item_counter))
+            thread = threading.Thread(target=_sub_thread_, args=(task_queue, result_queue, item_counter, progress_bar))
             thread.daemon = True
             thread.start()
             subs.append(thread)
     else:
         # Found this while digging around the multiprocessing API.
         # This might help some of the pickling errors when working with ssh
-        multiprocessing.allow_connection_pickling()
+        mpctx.allow_connection_pickling()
 
         # Adjust number of sub-processes to spawn.
         if pcount == 0:
@@ -356,7 +371,7 @@ def sshread(serverjobs, pcount=None, tcount=None, progress_bar=False):
         pcount = int(min(pcount, totaljobs))
 
         if pcount > cpuhardlimit():
-            raise ValueError('CPUHardLimit exceeded: ' + str(pcount) + ' > ' + str(cpuhardlimit()))
+            raise ValueError('CPUHardLimit exceeded: %d > %d' % (pcount, cpuhardlimit()))
 
         if tcount is None:
             tcount = 0
@@ -369,7 +384,7 @@ def sshread(serverjobs, pcount=None, tcount=None, progress_bar=False):
 
         log.info(u'spawning %d sub-processes' % (pcount, ))
         for pid in range(pcount):
-            pid = multiprocessing.Process(target=_sub_process_, args=(task_queue, result_queue, item_counter, tcount))
+            pid = mpctx.Process(target=_sub_process_, args=(task_queue, result_queue, item_counter, tcount, progress_bar))
             pid.daemon = True
             pid.start()
             subs.append(pid)
@@ -398,7 +413,7 @@ def sshread(serverjobs, pcount=None, tcount=None, progress_bar=False):
         return result_queue.get()
 
 
-def _sub_process_(task_queue, result_queue, item_counter, thread_count):
+def _sub_process_(task_queue, result_queue, item_counter, thread_count, progress_bar):
     """ Private method for managing multi-processing and spawning thread pools.
 
     DO NOT USE THIS METHOD!
@@ -410,13 +425,14 @@ def _sub_process_(task_queue, result_queue, item_counter, thread_count):
             job = task_queue.get()
             job.run()
             result_queue.put(job)
-            with item_counter.get_lock():
-                item_counter.value += 1
+            if progress_bar:
+                with item_counter.get_lock():
+                    item_counter.value += 1
     else:
         threads = list()
         log.debug(u'process: %d spawning: %d threads' % (pid, thread_count))
         for thread in range(thread_count):
-            thread = threading.Thread(target=_sub_thread_, args=(task_queue, result_queue, item_counter))
+            thread = threading.Thread(target=_sub_thread_, args=(task_queue, result_queue, item_counter, progress_bar))
             thread.daemon = True
             thread.start()
             threads.append(thread)
@@ -427,7 +443,7 @@ def _sub_process_(task_queue, result_queue, item_counter, thread_count):
     return None
 
 
-def _sub_thread_(task_queue, result_queue, item_counter):
+def _sub_thread_(task_queue, result_queue, item_counter, progress_bar):
     """ Private method for managing multi-processing and spawning thread pools.
 
     DO NOT USE THIS METHOD!
@@ -437,7 +453,8 @@ def _sub_thread_(task_queue, result_queue, item_counter):
         job = task_queue.get()
         job.run()
         result_queue.put(job)
-        with item_counter.get_lock():
-            item_counter.value += 1
+        if progress_bar:
+            with item_counter.get_lock():
+                item_counter.value += 1
     log.debug('existing thread')
     return None
